import pickle
import numpy as np
import pandas as pd
import seaborn as sns
import matplotlib.pyplot as plt
from scipy.stats import pearsonr, spearmanr
from sklearn.decomposition import PCA
from sklearn.cross_decomposition import CCA, PLSRegression
from sklearn.preprocessing import StandardScaler
from nilearn import datasets, surface, plotting
import sys
sys.path.append('./')
import cca_funcs as cf
import subprocess

# load data
main_dir = '.'
cbf_data = pickle.load(open(f'{main_dir}/cbfdata.pkl', 'rb'))
pet_data = pickle.load(open(f'{main_dir}/petdata.pkl', 'rb'))
subjects = pickle.load(open(f'{main_dir}/subjects.pkl', 'rb'))

# Generate PALM exchange blocks
eb = np.zeros([len(subjects),4])
eb[:,0] = -1
l = 1
k = 1
for i, subject in enumerate(subjects):
    score = []
    for j, subject2 in enumerate(subjects):
        if subject[:-2] in subject2:
            score.append(j)
    if len(score) == 2:
        eb[i, 1] = 1
        eb[score[0], 2] = k
        eb[score[1], 2] = k
        eb[score[0], 3] = 1
        eb[score[1], 3] = 2
        k += 1
    if len(score) == 1:
        eb[i, 1] = -1
        eb[i, 2] = 99
        eb[i, 3] = l
        l += 1

np.savetxt(f"{main_dir}/EB.csv", eb, delimiter=',')

# run PALM - requires installation of PALM toolbox https://fsl.fmrib.ox.ac.uk/fsl/fslwiki/PALM/ to currennt directory
subprocess.call('''matlab -nodisplay -nosplash -nodesktop -r "run('./generate_perms.m');exit;"''', shell=True)

# load permutations generated by PALM
perms = np.loadtxt('./perms.csv', delimiter=',')    

# Permute on HPC set up for SGE
inputs = {'pet_data': pet_data,
          'cbf_data': cbf_data,
          'components': (2, 11)}
slots = 20
num_perms = 10000
label = f'cca_palm'

# Send to queue
pf.queue_perm_test_cca_palm(inputs, label, slots, num_perms)

# get results once job complete
results_palm = cf.run_perm_cca(inputs, label, slots, num_perms)

# Plot the results of the permutation test
plt.rcParams['figure.figsize'] = [6, 6]
sns.kdeplot(results_palm['perm_list'], cut=0, label = 'Dependency Corrected \n Permutations')
plt.vlines(x=results_palm['true_mean'], ymin=0.1, ymax=1, color='dodgerblue')
plt.xlabel(r'Out of Sample Correlation (r$_p$)')
plt.ylabel('Frequency')
plt.legend()
sns.despine()

# Plot Full sample regression
# Fit CCA for 8 PCA comps on whole sample
pca = PCA(n_components=8, svd_solver='arpack')
cbf_pca = pca.fit_transform(StandardScaler().fit_transform(cbf_data))
pet_pca = pca.fit_transform(StandardScaler().fit_transform(pet_data))
cca_orig = CCA()
tf = cca_orig.fit_transform(cbf_pca, pet_pca)

# Dataframe labelling whether pla/amph session
amph = np.hstack([np.ones([1, 24]), np.zeros([1, 27])])
subject = ['sub-503','sub-506','sub-507','sub-508','sub-509','sub-510','sub-511','sub-512','sub-513','sub-514','sub-515','sub-516','sub-517','sub-518','sub-519','sub-520','sub-521','sub-523','sub-524','sub-525','sub-526','sub-529','sub-531','sub-533','sub-501','sub-502','sub-503','sub-505','sub-506','sub-507','sub-508','sub-510','sub-511','sub-512','sub-513','sub-514','sub-515','sub-516','sub-517','sub-518','sub-519','sub-520','sub-521','sub-522','sub-523','sub-524','sub-525','sub-526','sub-529','sub-531','sub-533']
data = np.vstack([tf[0][:, 0], tf[1][:, 0], amph, subject])
df = pd.DataFrame(data = data.T, columns = ['cbf_score', 'pet_score', 'amph', 'subject'])
df[["cbf_score", "pet_score"]] = df[["cbf_score", "pet_score"]].apply(pd.to_numeric)

plt.rcParams['figure.figsize'] = [4, 4]
ax = sns.regplot(x='cbf_score', y='pet_score', data=df, scatter=False)
ax = sns.scatterplot(x='cbf_score', y='pet_score', hue = 'amph', data=df)
plt.xlabel('CCA Scores: Cortical CBF')
plt.ylabel('CCA Scores: Striatal D2/3')
legend_labels, _ = ax.get_legend_handles_labels()
ax.legend(legend_labels[1:3], ['Placebo', 'Amph'], 
          bbox_to_anchor=(0.5, 1), 
          title='')
ax.set(xlim=(-2.25, 2.25))
sns.despine()

# CHANGE IN CBF AND CHANGE IN PET
d_cbf, d_pet, d_sub = [], [], []
for subject in df.subject.unique():
    subset = df[df['subject']==subject]
    if len(subset)>1:
        d_cbf.append(subset.cbf_score.iloc[0]-subset.cbf_score.iloc[1])
        d_pet.append(subset.pet_score.iloc[0]-subset.pet_score.iloc[1])
        d_sub.append(subject)
change_df = pd.DataFrame(data = np.array([d_cbf, d_pet]).T, columns=['Change in PET CCA Score', 'Change in CBF CCA Score'])
rs,ps = spearmanr(d_cbf, d_pet)
r,p = pearsonr(d_cbf, d_pet)

if ps<0.001:
    ps='<0.001'
else:
    ps=f'={str(ps)[:5]}'

plt.rcParams['figure.figsize'] = [4, 4]
ax = sns.regplot(x='Change in CBF CCA Score', y='Change in PET CCA Score', data=change_df)
ax = sns.scatterplot(x='Change in CBF CCA Score', y='Change in PET CCA Score', data=change_df, alpha=0.8)
plt.xlabel('Change in Cortical CBF CCA Score')
plt.ylabel('Change in Striatal D2/3 CCA Score')
ax.text(3.0, 2.0, f'r={str(rs)[:4]}')
ax.text(3.0, 1.75, f'p{ps}')
ax.set(xlim=(-0.5, 4.5))
plt.tight_layout()
sns.despine()

# relationship of CCA scores with subjective effects
symp_scores = [16.0,19.0,22.0,20.0,np.nan,12.0,17.0,21.0,13.0,15.0,23.0,19.0,19.0,12.0,12.0,16.0,30.0,37.0,17.0,18.0,14.0,12.0,20.0,25.0,10,24,10,12,10,12,16,10,13,10,11,10,17,10,10,10,10,15,11,14,11,10,11,12,10,10,15]
drug_status = 24*['amph']+27*['plac']
data = np.vstack([(cca_orig.x_scores_[:, 0]+cca_orig.y_scores_[:, 0])/2,
                  symp_scores, drug_status])
df = pd.DataFrame(data.T, columns = ['avg', 'symp', 'drug_status'])
df['avg'] = df['avg'].astype('float64')
df['symp'] = df['symp'].astype('float64')
df = df.dropna()
df = df.sort_values(by='drug_status', ascending=False)

plt.rcParams['figure.figsize']=[5,5]
ax = sns.scatterplot('avg', 'symp', hue='drug_status', palette = ['steelblue', 'darkorange'],data=df)
ax = sns.regplot(x='avg', y='symp', data=df[27:], color='darkorange', scatter=False)
ax = sns.regplot(x='avg', y='symp', data=df[:27], color='steelblue', scatter=False)
plt.xlabel('Mean CCA Score')
plt.ylabel('Subjective Effects of Amphetamine')
legend_labels, _ = ax.get_legend_handles_labels()
ax.legend(legend_labels[1:3], ['Placebo', 'Amphetamine'], 
          bbox_to_anchor=(1, 1), 
          title='')
ax.set(xlim=(-2.0, 2.5))
sns.despine()