import os
import numpy as np
from sklearn.cross_decomposition import CCA
from sklearn.preprocessing import StandardScaler
from sklearn.decomposition import PCA
import math
from scipy.stats import pearsonr
import inspect
import subprocess
import pickle
import sys
sys.path.append('./')
import cca_funcs as cf


def queue_perm_test_cca_palm(inputs, label, slots, num_perms):
    ''' runs out of sample permutations on cluster'''
    perm_dir = f'queue_{label}'
    os.mkdir(f'./{perm_dir}')

    # save the inputs dict to the dir
    f = open(f"./{perm_dir}/inputs.pkl","wb")
    pickle.dump(inputs,f)
    f.close()

    # save perm.list
    perms_per_slot = int(num_perms/slots) # must be round number
    lines = range(slots)
    with open(f'{perm_dir}/perm.list', 'w') as f: 
        f.write('\n'.join([str(line) for line in lines]))

    # save the Python script
    with open(f"./{perm_dir}/perm.py", "w") as python_file:
        script = """import pickle
        import sys
        import copy
        import numpy as np
        sys.path.append('.')
        import cca_funcs as cf
        from scipy.stats import pearsonr

        perm_num = int(sys.argv[1])
        perm_dir = sys.argv[2]
        perms_per_slot = int(sys.argv[3])


        # Load the inputs dict
        f = open(f"../{perm_dir}/inputs.pkl", 'rb')
        inputs = pickle.load(f)
        pet_data = inputs['pet_data']
        cbf_data = inputs['cbf_data']
        components = inputs['components']

        # permutation test
        perms = np.loadtxt('./perms.csv', delimiter=',')    
        np.random.seed(seed=perm_num)
        perm_rs_mean = []
        for perm in range(perms_per_slot):
            print(perm)
            shuffle = perms[:,np.random.choice(199999)+1] 
            shuffle = shuffle.astype(int)-1
            # np.random.shuffle(cbf_data)
            rs = cf.cca_cross_validate(cbf_data[shuffle], pet_data, components)
            perm_rs_mean.append(np.mean(rs))

        # save output
        f = open(f"./perm_results{perm_num}.pkl","wb")
        pickle.dump(perm_rs_mean,f)
        f.close()
        """
        script = inspect.cleandoc(script)
        python_file.write(script)
        python_file.close()
    
    # save the queue scipt
    with open(f"./{perm_dir}/perm.queue", "w") as queue_file:
        queue= """#$ -m n
                #$ -M example@example.ac.uk
                #$ -o /logs/perm_cca_o$TASK_ID
                #$ -e /logs/perm_cca_e$TASK_ID 
                #$ -q global 
                #$ -N perm_cca 
                #$ -pe smp 4
                setenv CONDA_ENV_PATH #add path to virtual environment containing the required packages
                setenv CONDA_DEFAULT_ENV #add name  of virtual environment
                setenv PATH {path to virtual env}/bin:${PATH}
                cd {code directory}}
                set list='./perm.list'
                set file="`awk 'FNR==$SGE_TASK_ID' ${list}`"

                python perm.py ${file} %s %s""" % (perm_dir, perm_dir, perms_per_slot)
        queue = inspect.cleandoc(queue)
        queue_file.write(queue)
        queue_file.close()    

    # submit to queue
    subprocess.run(f"qsub -t  1-{slots} {perm_dir}/perm.queue", shell=True, check=True)



def cca_cross_validate(cbf_data, pet_data, components):
    ''' Calculate out of sample pearson correlation between x/y scores
    for different numbers of PCA components. Returns the average correlation
    for 100 differen 80/20 test/train splits
    '''
    np.random.seed(seed=1)
    rs = []
    n = np.arange(len(cbf_data))  # number of subjects
    select = math.ceil(len(cbf_data)/5)  # Use 80/20 train/test split
    cbf_standard = StandardScaler().fit_transform(cbf_data)
    pet_standard = StandardScaler().fit_transform(pet_data)

    for n_componenets in range(components[0], components[1]):
        print(f'comp{n_componenets}')
        comp_r = []
        for i in range(100): # 100 seperate train/test splits
            print(i)
            test_idx = np.random.choice(n, select, replace=False)
            train_idx = np.delete(n, test_idx)

            # PCA only fit on training data 
            cbf_pca = PCA(n_components=n_componenets, svd_solver='arpack')
            cbf_pca_train = cbf_pca.fit_transform(cbf_standard[train_idx])
            pet_pca = PCA(n_components=n_componenets, svd_solver='arpack')
            pet_pca_train = pet_pca.fit_transform(pet_standard[train_idx])
            
            # Run CCA on training data
            cca_train = CCA()
            cca_train.fit(cbf_pca_train, pet_pca_train)

            # Use PCA and CCA models fit on training data on test data
            cbf_pca_test = cbf_pca.transform(cbf_standard[test_idx])
            pet_pca_test = pet_pca.transform(pet_standard[test_idx])
            test_results = cca_train.transform(cbf_pca_test, pet_pca_test)
            r = pearsonr(test_results[0][:,0], test_results[1][:,0])[0]
            comp_r.append(r)
        rs.append(np.mean(comp_r))
    return rs


def run_perm_cca(inputs, label, slots, num_perms):
    '''waits to see if clsuter permutations have been run
    by  and then calculates observed values and p values'''
    perm_dir = f'queue_{label}'

    # run permutation testing on cluster
    # queue_perm_test(inputs, label, slots, num_perms)
    cbf_data = inputs['cbf_data']
    pet_data = inputs['pet_data']
    components = inputs['components']

    # get true value
    np.random.seed(seed=1)
    true_rs = cf.cca_cross_validate(cbf_data, pet_data, components)
    true_mean_r = np.mean(true_rs)

    # check permutation analysis complete
    waiting = 0
    while waiting < slots:
        complete=[]
        for i in range(slots):
            complete.append(os.path.exists(f"{perm_dir}/perm_results{i}.pkl"))
        waiting = np.sum(complete)
    
    #load pickles
    group_mean_pickle = []

    for i in range(slots):
        loaded_pickle = pickle.load(open(f"{perm_dir}/perm_results{i}.pkl", "rb"))
        group_mean_pickle = group_mean_pickle + loaded_pickle[1]

    # get p
    mean_p = np.sum(true_mean_r<=np.array(group_mean_pickle))/num_perms
    final_results = {'true_mean': true_mean_r,
                     'mean_p': mean_p,
                     'perm_list': group_mean_pickle}
    return final_results